"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const command_1 = require("@heroku-cli/command");
const create_addon_1 = __importDefault(require("../../../lib/addons/create_addon"));
const notify_1 = __importDefault(require("../../../lib/notify"));
const inquirer_1 = require("inquirer");
const color_1 = __importDefault(require("@heroku-cli/color"));
const baseCommand_1 = __importDefault(require("../../../lib/baseCommand"));
class Create extends baseCommand_1.default {
    static description = 'Create a postgres performance addon';
    static examples = ['<%= config.bin %> <%= command.id %> performance --level 4G-Performance'];
    static args = {
        tier: core_1.Args.string({
            description: 'Postgres tier',
            options: ['performance'],
            required: true,
        }),
    };
    static flags = {
        level: command_1.flags.string({
            description: 'Set compute scale',
        }),
        name: command_1.flags.string({ description: 'Name for the add-on resource' }),
        as: command_1.flags.string({ description: 'Name for the initial add-on attachment' }),
        wait: command_1.flags.boolean({ description: 'Watch add-on creation status and exit when complete' }),
        'high-availability': command_1.flags.boolean({
            description: 'Enable a high availability cluster by provisioning a warm standby writer.',
            allowNo: true,
        }),
        network: command_1.flags.string({ description: 'Networking for the add-on', options: ['private', 'shield'] }),
        app: command_1.flags.app({
            required: true,
        }),
        readers: command_1.flags.integer({ description: 'Provisions a reader instance pool with the specified number of instances', min: 0, max: 15 }),
    };
    async run() {
        const { flags } = await this.parse(Create);
        const { app, name, as, wait, confirm, network } = flags;
        let { level, readers } = flags;
        let highAvailability = flags['high-availability'];
        if (!level) {
            const res = await this.dataApi.get('/data/postgres/v1/levels/performance');
            const body = res.body;
            level = (await (0, inquirer_1.prompt)({
                name: 'level',
                type: 'list',
                message: 'Select a level',
                choices: body.map((level) => level.name),
                pageSize: 12,
            })).level;
        }
        if (readers === undefined) {
            readers = Number((await (0, inquirer_1.prompt)({
                name: 'readers',
                type: 'input',
                message: 'Select number of instances for a reader instance pool',
                validate: (value) => {
                    const parsedValue = parseInt(value);
                    if (isNaN(parsedValue)) {
                        return 'Please enter a valid integer';
                    }
                    return true;
                },
                default: 0,
            })).readers);
        }
        const service = process.env.HEROKU_DATA_SERVICE || 'heroku-postgresql';
        const servicePlan = `${service}:performance-${network ? `${network}-` : ''}beta`;
        const config = { level: level, "high-availability": highAvailability, followers: readers ? String(readers) : undefined };
        let addon;
        try {
            core_1.ux.action.start(`Creating a ${color_1.default.cyan(level)} database on ${color_1.default.app(app)}`);
            addon = await (0, create_addon_1.default)(this.heroku, app, servicePlan, confirm, wait, { config, name, as });
            if (wait) {
                (0, notify_1.default)(`heroku addons:create ${addon.name}`, 'Add-on successfully provisioned');
            }
            core_1.ux.action.stop();
        }
        catch (error) {
            if (wait) {
                (0, notify_1.default)(`heroku addons:create ${servicePlan}`, 'Add-on failed to provision', false);
            }
            throw error;
        }
    }
}
exports.default = Create;
