"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ago = ago;
const core_1 = require("@oclif/core");
const command_1 = require("@heroku-cli/command");
const color_1 = __importDefault(require("@heroku-cli/color"));
const resolveAddon_1 = require("../../../lib/data/pg/resolveAddon");
const baseCommand_1 = __importDefault(require("../../../lib/baseCommand"));
const utils_1 = require("../../../lib/utils");
/**
 * @description Returns human-readable relative time since the date. This code is inspired by ps.
 * @example
 * const date = new Date();
 * date.setHours(date.getHours() - 1);
 *
 * ago(date)
 *
 * // returns
 *
 * "(~ 1h ago)"
 */
function ago(since) {
    const elapsed = Math.floor((Date.now() - since.getTime()) / 1000);
    if (elapsed < 60)
        return `(~ ${Math.floor(elapsed)}s ago)`;
    if (elapsed < 60 * 60)
        return `(~ ${Math.floor(elapsed / 60)}m ago)`;
    if (elapsed < 60 * 60 * 25)
        return `(~ ${Math.floor(elapsed / 60 / 60)}h ago)`;
    return `(~ ${Math.floor(elapsed / 60 / 60 / 24)}d ago)`;
}
function getPoolsSummary(pools, haWriter) {
    const summary = pools.reduce((acc, pool, poolIdx) => {
        const poolName = pool.status === "Available" ? `${color_1.default.green(pool.name)}` : `${color_1.default.yellow(pool.name)}`;
        const poolStatus = pool.status === "Available" ? `(${color_1.default.green(pool.status)})` : `(${color_1.default.yellow(pool.status)})`;
        const renderedHa = pool.name === 'writer' && haWriter ? `(${color_1.default.cyan('HA')})` : '';
        const poolSize = `${color_1.default.cyan(pool.instances.length)} @ ${color_1.default.cyan(pool.level)}${renderedHa}:`;
        acc.headers[poolIdx] = [poolName, poolStatus, poolSize, pool.attachment_name];
        acc.details[poolIdx] = [];
        pool.instances.forEach((instance, i) => {
            let instanceName;
            if (instance.role == 'standby_writer') {
                instanceName = color_1.default.grey(instance.name);
            }
            else {
                instanceName = instance.name;
            }
            const status = instance.status === 'up' ? color_1.default.green(instance.status) : color_1.default.yellow(instance.status);
            const date = color_1.default.dim(ago(new Date(instance.updated_at)));
            acc.details[poolIdx].push([`${instanceName}:`, status, date]);
        });
        return acc;
    }, { headers: [], details: [] });
    const alignedHeaders = (0, utils_1.alignColumns)(summary.headers);
    const alignedDetails = summary.details.map(poolDetails => (0, utils_1.alignColumns)(poolDetails));
    return { alignedHeaders, alignedDetails };
}
class Info extends baseCommand_1.default {
    static description = 'Get Postgres Addon Info';
    static examples = ['<%= config.bin %> <%= command.id %> addon_name'];
    static args = {
        addon: core_1.Args.string({
            description: 'Name of the addon, its attachment name, or related config var on an app',
            required: true,
        }),
    };
    static flags = {
        app: command_1.flags.app(),
    };
    async run() {
        const { args, flags } = await this.parse(Info);
        const { addon: addonName } = args;
        const addon = await (0, resolveAddon_1.resolveAddon)(this.heroku, addonName, flags.app);
        const res = await this.dataApi.get(`/data/postgres/v1/${addon.id}`);
        const body = res.body;
        const pools = body.pools;
        core_1.ux.styledHeader(`${color_1.default.addon(body.addon_name)} as ${color_1.default.attachment(body.attachment_name)} on ${color_1.default.app(body.app_name)}`);
        core_1.ux.styledObject({
            Tier: body.tier,
            App: body.app_name,
            Version: body.version,
            Status: body.status,
            "High Availability": body.highly_available === true ? "On" : "Off",
            Created: new Date(body.created_at).toLocaleString(),
            Tables: `${body.tables.used_message} (${body.tables.compliance_message})`,
            Data: `${body.data.used_message} (${body.data.compliance_message})`,
        }, ['Tier', 'App', 'Version', 'Status', 'High Availability', 'Created', 'Tables', 'Data']);
        if (pools.length) {
            const poolSummary = getPoolsSummary(pools, body.highly_available);
            core_1.ux.styledHeader("Instance Pools");
            poolSummary.alignedHeaders.forEach((header, poolIdx) => {
                core_1.ux.styledHeader(header);
                poolSummary.alignedDetails[poolIdx].forEach((instanceDetails, i) => {
                    core_1.ux.log(instanceDetails);
                });
                core_1.ux.log('');
            });
        }
    }
}
exports.default = Info;
