"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.env = exports.getBastion = void 0;
exports.fetchConfig = fetchConfig;
exports.getConfigs = getConfigs;
exports.tunnelConfig = tunnelConfig;
exports.sshTunnel = sshTunnel;
const core_1 = require("@oclif/core");
const debug_1 = __importDefault(require("debug"));
const node_events_1 = __importDefault(require("node:events"));
const node_util_1 = require("node:util");
const tunnel_ssh_1 = __importDefault(require("tunnel-ssh"));
const host_1 = __importDefault(require("./host"));
const pgDebug = (0, debug_1.default)('pg');
async function fetchConfig(heroku, db) {
    return heroku.get(`/client/v11/databases/${encodeURIComponent(db.id)}/bastion`, {
        hostname: (0, host_1.default)(),
    });
}
const getBastion = function (config, baseName) {
    // If there are bastions, extract a host and a key
    // otherwise, return an empty Object
    // If there are bastions:
    // * there should be one *_BASTION_KEY
    // * pick one host from the comma-separated list in *_BASTIONS
    // We assert that _BASTIONS and _BASTION_KEY always exist together
    // If either is falsy, pretend neither exist
    const bastionKey = config[`${baseName}_BASTION_KEY`];
    const bastions = (config[`${baseName}_BASTIONS`] || '').split(',');
    const bastionHost = bastions[Math.floor(Math.random() * bastions.length)];
    return (bastionKey && bastionHost) ? { bastionHost, bastionKey } : {};
};
exports.getBastion = getBastion;
function getConfigs(db) {
    const dbEnv = (0, exports.env)(db);
    const dbTunnelConfig = tunnelConfig(db);
    if (db.bastionKey) {
        Object.assign(dbEnv, {
            PGPORT: dbTunnelConfig.localPort,
            PGHOST: dbTunnelConfig.localHost,
        });
    }
    return {
        dbEnv,
        dbTunnelConfig,
    };
}
const env = (db) => {
    const baseEnv = Object.assign({
        PGAPPNAME: 'psql non-interactive',
        PGSSLMODE: (!db.host || db.host === 'localhost') ? 'prefer' : 'require',
    }, process.env);
    const mapping = {
        PGUSER: 'user',
        PGPASSWORD: 'password',
        PGDATABASE: 'database',
        PGPORT: 'port',
        PGHOST: 'host',
    };
    Object.keys(mapping).forEach(envVar => {
        const val = db[mapping[envVar]];
        if (val) {
            baseEnv[envVar] = val;
        }
    });
    return baseEnv;
};
exports.env = env;
function tunnelConfig(db) {
    const localHost = '127.0.0.1';
    const localPort = Math.floor((Math.random() * (65535 - 49152)) + 49152);
    return {
        username: 'bastion',
        host: db.bastionHost,
        privateKey: db.bastionKey,
        dstHost: db.host || undefined,
        dstPort: (db.port && Number.parseInt(db.port, 10)) || undefined,
        localHost,
        localPort,
    };
}
/* SEPARATOR */
class Timeout {
    timeout;
    message;
    events = new node_events_1.default();
    timer;
    constructor(timeout, message) {
        this.timeout = timeout;
        this.message = message;
    }
    async promise() {
        this.timer = setTimeout(() => {
            this.events.emit('error', new Error(this.message));
        }, this.timeout);
        try {
            await node_events_1.default.once(this.events, 'cancelled');
        }
        finally {
            clearTimeout(this.timer);
        }
    }
    cancel() {
        this.events.emit('cancelled');
    }
}
async function sshTunnel(db, dbTunnelConfig, timeout = 10000) {
    if (!db.bastionKey) {
        return null;
    }
    const timeoutInstance = new Timeout(timeout, 'Establishing a secure tunnel timed out');
    const createSSHTunnel = (0, node_util_1.promisify)(tunnel_ssh_1.default);
    try {
        return await Promise.race([
            timeoutInstance.promise(),
            createSSHTunnel(dbTunnelConfig),
        ]);
    }
    catch (error) {
        pgDebug(error);
        core_1.ux.error('Unable to establish a secure tunnel to your database.');
    }
    finally {
        timeoutInstance.cancel();
    }
}
