"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAttachment = getAttachment;
exports.database = database;
const color_1 = __importDefault(require("@heroku-cli/color"));
const api_client_1 = require("@heroku-cli/command/lib/api-client");
const core_1 = require("@oclif/core");
const debug_1 = __importDefault(require("debug"));
const bastion_1 = require("./bastion");
const config_1 = require("./config");
const resolve_1 = require("./resolve");
const util_1 = require("./util");
const util_2 = require("../../../addons/util");
const pgDebug = (0, debug_1.default)('pg');
async function matchesHelper(heroku, app, db, namespace) {
    pgDebug(`fetching ${db} on ${app}`);
    const addonService = process.env.HEROKU_POSTGRESQL_ADDON_NAME || process.env.HEROKU_DATA_SERVICE || 'heroku-postgresql';
    pgDebug(`addon service: ${addonService}`);
    try {
        const attached = await (0, resolve_1.appAttachment)(heroku, app, db, { addon_service: addonService, namespace });
        return ({ matches: [attached] });
    }
    catch (error) {
        if (error instanceof resolve_1.AmbiguousError && error.body?.id === 'multiple_matches' && error.matches) {
            return { matches: error.matches, error };
        }
        if (error instanceof api_client_1.HerokuAPIError && error.http.statusCode === 404 && error.body && error.body.id === 'not_found') {
            return { matches: null, error };
        }
        throw error;
    }
}
async function getAttachment(heroku, app, db = 'DATABASE_URL', namespace = '') {
    const matchesOrError = await matchesHelper(heroku, app, db, namespace);
    let { matches } = matchesOrError;
    const { error } = matchesOrError;
    // happy path where the resolver matches just one
    if (matches && matches.length === 1) {
        return matches[0];
    }
    // case for 404 where there are implicit attachments
    if (!matches) {
        const appConfigMatch = /^(.+?)::(.+)/.exec(db);
        if (appConfigMatch) {
            app = appConfigMatch[1];
            db = appConfigMatch[2];
        }
        if (!db.endsWith('_URL')) {
            db += '_URL';
        }
        const [config = {}, attachments] = await Promise.all([
            (0, config_1.getConfig)(heroku, app),
            allAttachments(heroku, app),
        ]);
        if (attachments.length === 0) {
            throw new Error(`${color_1.default.app(app)} has no databases`);
        }
        matches = attachments.filter(attachment => config[db] && config[db] === config[(0, util_1.getConfigVarName)(attachment.config_vars)]);
        if (matches.length === 0) {
            const validOptions = attachments.map(attachment => (0, util_1.getConfigVarName)(attachment.config_vars));
            throw new Error(`Unknown database: ${db}. Valid options are: ${validOptions.join(', ')}`);
        }
    }
    // case for multiple attachments with passedDb
    const first = matches[0];
    // case for 422 where there are ambiguous attachments that are equivalent
    if (matches.every(match => first.addon?.id === match.addon?.id && first.app?.id === match.app?.id)) {
        const config = await (0, config_1.getConfig)(heroku, first.app.name) ?? {};
        if (matches.every(match => config[(0, util_1.getConfigVarName)(first.config_vars)] === config[(0, util_1.getConfigVarName)(match.config_vars)])) {
            return first;
        }
    }
    throw error;
}
async function allAttachments(heroku, app_id) {
    const { body: attachments } = await heroku.get(`/apps/${app_id}/addon-attachments`, {
        headers: { 'Accept-Inclusion': 'addon:plan,config_vars' },
    });
    return attachments.filter((a) => a.addon.plan?.name?.startsWith('heroku-postgresql'));
}
async function database(heroku, app, db, namespace, pool) {
    const attached = await getAttachment(heroku, app, db, namespace);
    if (pool && !(0, util_2.performancePgPlan)(attached.addon))
        core_1.ux.error('‘--pool’ flag is only supported for Performance-tier databases', { exit: 1 });
    // would inline this as well but in some cases attachment pulls down config
    // as well, and we would request twice at the same time but I did not want
    // to push this down into attachment because we do not always need config
    const config = await (0, config_1.getConfig)(heroku, attached.app.name);
    const database = (0, util_1.getConnectionDetails)(attached, config, pool);
    if ((0, util_1.bastionKeyPlan)(attached.addon) && !database.bastionKey) {
        const { body: bastionConfig } = await (0, bastion_1.fetchConfig)(heroku, attached.addon);
        const bastionHost = bastionConfig.host;
        const bastionKey = bastionConfig.private_key;
        Object.assign(database, { bastionHost, bastionKey });
    }
    return database;
}
